	list	p=16f84a
	include <p16f84a.inc>

	__CONFIG   _CP_OFF & _WDT_OFF & _XT_OSC

SongPos	equ  0C
PatPos	equ   0D
SongVal	equ  0E
ToneVal	equ  0F
ToneLen	equ  10
BeepBit	equ  11
LoopVal	equ  12
Tmp1	equ  13
Tmp2	equ  14
TmpPat	equ  15


	org 0x000

Init
	clrf	PORTB
	bsf	STATUS,RP0		; select bank1
	movlw	h'FE'			; set PORTB bit0 to output mode
	movwf	TRISB
	bcf	STATUS,RP0		; select bank0

	clrf	SongPos
	clrf	PatPos
	clrf	BeepBit

Music
	call	PatternTab		; get current song value
	movwf	SongVal
	clrf	PCLATH
	andwf	SongVal,w		; test for zero
	btfsc	STATUS,Z
	goto	LongWait
	sublw	0xFF			; 0xFF = end of pattern
	btfsc	STATUS,Z
	goto	NextPattern
	movf	SongVal,w
	call	ToneValTab		; get tone value
	movwf	ToneVal
	movf	SongVal,w
	call	ToneLenTab		; get tone length
	movwf	ToneLen

ToneLoop
	movf	BeepBit,w
	movwf	PORTB
	xorlw	0x01			; toggle beeper
	movwf	BeepBit
	movf	ToneVal,w
	movwf	LoopVal
WaitLoop
	nop
	nop
	nop
	nop
	decfsz	LoopVal,f
	goto	WaitLoop
	nop
	nop
	nop
	nop
	decfsz	ToneLen,f
	goto	ToneLoop

; waits 29ms
; 20ms = 4nop, Tmp1=0x0D
; 29ms = 3nop, Tmp1=0x13
LongWait
	movlw	0x13
	movwf	Tmp1
Loop1
	clrf	Tmp2
Loop2
	nop
	nop
	nop
	decfsz	Tmp2,f
	goto	Loop2
	decfsz	Tmp1,f
	goto	Loop1

	incf	SongPos,f
	goto	Music

NextPattern
	clrf	SongPos			; reset song pointer to the beginning of the next pattern
	incf	PatPos,w
	movwf	PatPos
	sublw	0x12			; music has 18 patterns
	btfsc	STATUS,Z
	clrf	PatPos			; restart music if end of patterns reached
	goto	Music			; or else continue playing music

PatternTab
	bcf	STATUS,C		; rotate uses carry flag!
	rlf	PatPos,w
	movwf	TmpPat			; one entry is 4 word wide
	rlf	TmpPat,w		; so multiply w by 4
	addwf	PCL,f			; computed goto to the selected element

	movlw	0x01			; table is in Page 1
	movwf	PCLATH
	movf	SongPos,w
	goto	PatTab1			; continue in PatTab1 to get pattern value

	movlw	0x01
	movwf	PCLATH
	movf	SongPos,w
	goto	PatTab2

	movlw	0x01
	movwf	PCLATH
	movf	SongPos,w
	goto	PatTab1

	movlw	0x01
	movwf	PCLATH
	movf	SongPos,w
	goto	PatTab2

	movlw	0x01
	movwf	PCLATH
	movf	SongPos,w
	goto	PatTab3

	movlw	0x01
	movwf	PCLATH
	movf	SongPos,w
	goto	PatTab3

	movlw	0x01
	movwf	PCLATH
	movf	SongPos,w
	goto	PatTab4

	movlw	0x01
	movwf	PCLATH
	movf	SongPos,w
	goto	PatTab4

	movlw	0x01
	movwf	PCLATH
	movf	SongPos,w
	goto	PatTab1

	movlw	0x02
	movwf	PCLATH
	movf	SongPos,w
	goto	PatTab5

	movlw	0x02
	movwf	PCLATH
	movf	SongPos,w
	goto	PatTab6

	movlw	0x02
	movwf	PCLATH
	movf	SongPos,w
	goto	PatTab7

	movlw	0x02
	movwf	PCLATH
	movf	SongPos,w
	goto	PatTab6

	movlw	0x02
	movwf	PCLATH
	movf	SongPos,w
	goto	PatTab8

	movlw	0x02
	movwf	PCLATH
	movf	SongPos,w
	goto	PatTab6

	movlw	0x02
	movwf	PCLATH
	movf	SongPos,w
	goto	PatTab7

	movlw	0x02
	movwf	PCLATH
	movf	SongPos,w
	goto	PatTab6

	movlw	0x02
	movwf	PCLATH
	movf	SongPos,w
	goto	PatTab8


ToneValTab
	addwf	PCL,f
	retlw	0xFF	; line: 0
	retlw	0xF0	; line: 1
	retlw	0xE3	; line: 2
	retlw	0xD6	; line: 3
	retlw	0xCA	; line: 4
	retlw	0xBE	; line: 5
	retlw	0xB3	; line: 6
	retlw	0xA9	; line: 7
	retlw	0x9F	; line: 8
	retlw	0x96	; line: 9
	retlw	0x8E	; line: 10
	retlw	0x86	; line: 11
	retlw	0x7E	; line: 12
	retlw	0x77	; line: 13
	retlw	0x70	; line: 14
	retlw	0x6A	; line: 15
	retlw	0x64	; line: 16
	retlw	0x5E	; line: 17
	retlw	0x58	; line: 18
	retlw	0x53	; line: 19
	retlw	0x4E	; line: 20
	retlw	0x4A	; line: 21
	retlw	0x46	; line: 22
	retlw	0x42	; line: 23
	retlw	0x3E	; line: 24
	retlw	0x3A	; line: 25
	retlw	0x37	; line: 26
	retlw	0x34	; line: 27
	retlw	0x30	; line: 28
	retlw	0x2E	; line: 29
	retlw	0x2B	; line: 30


ToneLenTab
	addwf	PCL,f
	retlw	0x0A	; line: 0
	retlw	0x0B	; line: 1
	retlw	0x0C	; line: 2
	retlw	0x0C	; line: 3
	retlw	0x0D	; line: 4
	retlw	0x0E	; line: 5
	retlw	0x0F	; line: 6
	retlw	0x10	; line: 7
	retlw	0x11	; line: 8
	retlw	0x12	; line: 9
	retlw	0x13	; line: 10
	retlw	0x14	; line: 11
	retlw	0x15	; line: 12
	retlw	0x16	; line: 13
	retlw	0x17	; line: 14
	retlw	0x19	; line: 15
	retlw	0x1A	; line: 16
	retlw	0x1C	; line: 17
	retlw	0x1E	; line: 18
	retlw	0x1F	; line: 19
	retlw	0x21	; line: 20
	retlw	0x23	; line: 21
	retlw	0x25	; line: 22
	retlw	0x27	; line: 23
	retlw	0x2A	; line: 24
	retlw	0x2C	; line: 25
	retlw	0x2F	; line: 26
	retlw	0x31	; line: 27
	retlw	0x35	; line: 28
	retlw	0x37	; line: 29
	retlw	0x3B	; line: 30


	org 0x100

PatTab1
	addwf	PCL,f	; line 1
	retlw	0x04	; line 2
	retlw	0x00	; line 3
	retlw	0x02	; line 4
	retlw	0x00	; line 5
	retlw	0x01	; line 6
	retlw	0x00	; line 7
	retlw	0x02	; line 8
	retlw	0x00	; line 9
	retlw	0x05	; line 10
	retlw	0x11	; line 11
	retlw	0x05	; line 12
	retlw	0x11	; line 13
	retlw	0x05	; line 14
	retlw	0x11	; line 15
	retlw	0x05	; line 16
	retlw	0x11	; line 17
	retlw	0x07	; line 18
	retlw	0x00	; line 19
	retlw	0x05	; line 20
	retlw	0x00	; line 21
	retlw	0x04	; line 22
	retlw	0x00	; line 23
	retlw	0x05	; line 24
	retlw	0x00	; line 25
	retlw	0x09	; line 26
	retlw	0x15	; line 27
	retlw	0x09	; line 28
	retlw	0x15	; line 29
	retlw	0x09	; line 30
	retlw	0x15	; line 31
	retlw	0x09	; line 32
	retlw	0x15	; line 33
	retlw	0x0A	; line 34
	retlw	0x00	; line 35
	retlw	0x09	; line 36
	retlw	0x00	; line 37
	retlw	0x08	; line 38
	retlw	0x00	; line 39
	retlw	0x09	; line 40
	retlw	0x00	; line 41
	retlw	0x10	; line 42
	retlw	0x00	; line 43
	retlw	0x0E	; line 44
	retlw	0x00	; line 45
	retlw	0x0D	; line 46
	retlw	0x00	; line 47
	retlw	0x0E	; line 48
	retlw	0x00	; line 49
	retlw	0x10	; line 50
	retlw	0x00	; line 51
	retlw	0x0E	; line 52
	retlw	0x00	; line 53
	retlw	0x0D	; line 54
	retlw	0x00	; line 55
	retlw	0x0E	; line 56
	retlw	0x00	; line 57
	retlw	0x11	; line 58
	retlw	0x1D	; line 59
	retlw	0x11	; line 60
	retlw	0x1D	; line 61
	retlw	0x11	; line 62
	retlw	0x1D	; line 63
	retlw	0x11	; line 64
	retlw	0x1D	; line 65
	retlw	0xFF	; line 66

PatTab2
	addwf	PCL,f	; line 67
	retlw	0x0E	; line 68
	retlw	0x00	; line 69
	retlw	0x00	; line 70
	retlw	0x00	; line 71
	retlw	0x11	; line 72
	retlw	0x00	; line 73
	retlw	0x00	; line 74
	retlw	0x00	; line 75
	retlw	0x10	; line 76
	retlw	0x0C	; line 77
	retlw	0x10	; line 78
	retlw	0x0C	; line 79
	retlw	0x10	; line 80
	retlw	0x0C	; line 81
	retlw	0x0E	; line 82
	retlw	0x00	; line 83
	retlw	0x0C	; line 84
	retlw	0x09	; line 85
	retlw	0x0C	; line 86
	retlw	0x00	; line 87
	retlw	0x0E	; line 88
	retlw	0x0C	; line 89
	retlw	0x0E	; line 90
	retlw	0x00	; line 91
	retlw	0x10	; line 92
	retlw	0x0C	; line 93
	retlw	0x10	; line 94
	retlw	0x0C	; line 95
	retlw	0x0E	; line 96
	retlw	0x0B	; line 97
	retlw	0x0E	; line 98
	retlw	0x00	; line 99
	retlw	0x0C	; line 100
	retlw	0x09	; line 101
	retlw	0x0C	; line 102
	retlw	0x00	; line 103
	retlw	0x0E	; line 104
	retlw	0x0C	; line 105
	retlw	0x0E	; line 106
	retlw	0x00	; line 107
	retlw	0x10	; line 108
	retlw	0x0C	; line 109
	retlw	0x10	; line 110
	retlw	0x00	; line 111
	retlw	0x0E	; line 112
	retlw	0x0B	; line 113
	retlw	0x0E	; line 114
	retlw	0x00	; line 115
	retlw	0x0C	; line 116
	retlw	0x09	; line 117
	retlw	0x0C	; line 118
	retlw	0x00	; line 119
	retlw	0x0B	; line 120
	retlw	0x08	; line 121
	retlw	0x0B	; line 122
	retlw	0x00	; line 123
	retlw	0x09	; line 124
	retlw	0x15	; line 125
	retlw	0x09	; line 126
	retlw	0x0C	; line 127
	retlw	0x10	; line 128
	retlw	0x14	; line 129
	retlw	0x15	; line 130
	retlw	0x00	; line 131
	retlw	0xFF	; line 132

PatTab3
	addwf	PCL,f	; line 133
	retlw	0x15	; line 134
	retlw	0x11	; line 135
	retlw	0x15	; line 136
	retlw	0x11	; line 137
	retlw	0x16	; line 138
	retlw	0x13	; line 139
	retlw	0x16	; line 140
	retlw	0x13	; line 141
	retlw	0x18	; line 142
	retlw	0x15	; line 143
	retlw	0x18	; line 144
	retlw	0x00	; line 145
	retlw	0x18	; line 146
	retlw	0x15	; line 147
	retlw	0x18	; line 148
	retlw	0x00	; line 149
	retlw	0x1A	; line 150
	retlw	0x16	; line 151
	retlw	0x18	; line 152
	retlw	0x15	; line 153
	retlw	0x16	; line 154
	retlw	0x13	; line 155
	retlw	0x15	; line 156
	retlw	0x11	; line 157
	retlw	0x13	; line 158
	retlw	0x10	; line 159
	retlw	0x13	; line 160
	retlw	0x10	; line 161
	retlw	0x13	; line 162
	retlw	0x10	; line 163
	retlw	0x13	; line 164
	retlw	0x10	; line 165
	retlw	0xFF	; line 166

PatTab4
	addwf	PCL,f	; line 167
	retlw	0x11	; line 168
	retlw	0x0E	; line 169
	retlw	0x11	; line 170
	retlw	0x0E	; line 171
	retlw	0x13	; line 172
	retlw	0x10	; line 173
	retlw	0x13	; line 174
	retlw	0x10	; line 175
	retlw	0x15	; line 176
	retlw	0x11	; line 177
	retlw	0x15	; line 178
	retlw	0x00	; line 179
	retlw	0x15	; line 180
	retlw	0x11	; line 181
	retlw	0x15	; line 182
	retlw	0x00	; line 183
	retlw	0x16	; line 184
	retlw	0x13	; line 185
	retlw	0x15	; line 186
	retlw	0x11	; line 187
	retlw	0x13	; line 188
	retlw	0x10	; line 189
	retlw	0x11	; line 190
	retlw	0x0E	; line 191
	retlw	0x10	; line 192
	retlw	0x0D	; line 193
	retlw	0x10	; line 194
	retlw	0x0D	; line 195
	retlw	0x10	; line 196
	retlw	0x0D	; line 197
	retlw	0x10	; line 198
	retlw	0x0D	; line 199
	retlw	0xFF	; line 200

	org 0x200

PatTab5
	addwf	PCL,f	; line 201
	retlw	0x1A	; line 202
	retlw	0x0E	; line 203
	retlw	0x1A	; line 204
	retlw	0x0E	; line 205
	retlw	0x1C	; line 206
	retlw	0x10	; line 207
	retlw	0x1C	; line 208
	retlw	0x10	; line 209
	retlw	0x1D	; line 210
	retlw	0x11	; line 211
	retlw	0x1D	; line 212
	retlw	0x11	; line 213
	retlw	0x1C	; line 214
	retlw	0x10	; line 215
	retlw	0x1C	; line 216
	retlw	0x10	; line 217
	retlw	0x1A	; line 218
	retlw	0x0E	; line 219
	retlw	0x1A	; line 220
	retlw	0x0E	; line 221
	retlw	0x19	; line 222
	retlw	0x0D	; line 223
	retlw	0x19	; line 224
	retlw	0x0D	; line 225
	retlw	0x1A	; line 226
	retlw	0x0E	; line 227
	retlw	0x1A	; line 228
	retlw	0x0E	; line 229
	retlw	0x15	; line 230
	retlw	0x09	; line 231
	retlw	0x15	; line 232
	retlw	0x09	; line 233
	retlw	0x16	; line 234
	retlw	0x0A	; line 235
	retlw	0x16	; line 236
	retlw	0x0A	; line 237
	retlw	0x13	; line 238
	retlw	0x07	; line 239
	retlw	0x13	; line 240
	retlw	0x07	; line 241
	retlw	0x11	; line 242
	retlw	0x1D	; line 243
	retlw	0x1A	; line 244
	retlw	0x15	; line 245
	retlw	0x11	; line 246
	retlw	0x1D	; line 247
	retlw	0x1A	; line 248
	retlw	0x15	; line 249
	retlw	0x10	; line 250
	retlw	0x1C	; line 251
	retlw	0x19	; line 252
	retlw	0x13	; line 253
	retlw	0x10	; line 254
	retlw	0x1C	; line 255
	retlw	0x19	; line 256
	retlw	0x13	; line 257
	retlw	0x0E	; line 258
	retlw	0x1A	; line 259
	retlw	0x15	; line 260
	retlw	0x11	; line 261
	retlw	0x0E	; line 262
	retlw	0x00	; line 263
	retlw	0x00	; line 264
	retlw	0x00	; line 265
	retlw	0xFF	; line 266

PatTab6
	addwf	PCL,f	; line 267
	retlw	0x1A	; line 268
	retlw	0x0E	; line 269
	retlw	0x1A	; line 270
	retlw	0x0E	; line 271
	retlw	0x1C	; line 272
	retlw	0x10	; line 273
	retlw	0x1C	; line 274
	retlw	0x10	; line 275
	retlw	0x1E	; line 276
	retlw	0x1A	; line 277
	retlw	0x15	; line 278
	retlw	0x12	; line 279
	retlw	0x0E	; line 280
	retlw	0x09	; line 281
	retlw	0x06	; line 282
	retlw	0x02	; line 283
	retlw	0x1A	; line 284
	retlw	0x0E	; line 285
	retlw	0x1A	; line 286
	retlw	0x0E	; line 287
	retlw	0x1C	; line 288
	retlw	0x10	; line 289
	retlw	0x1C	; line 290
	retlw	0x10	; line 291
	retlw	0x1E	; line 292
	retlw	0x12	; line 293
	retlw	0x1E	; line 294
	retlw	0x12	; line 295
	retlw	0x1E	; line 296
	retlw	0x12	; line 297
	retlw	0x1C	; line 298
	retlw	0x10	; line 299
	retlw	0x1A	; line 300
	retlw	0x0E	; line 301
	retlw	0x1A	; line 302
	retlw	0x0E	; line 303
	retlw	0x19	; line 304
	retlw	0x0D	; line 305
	retlw	0x19	; line 306
	retlw	0x0D	; line 307
	retlw	0xFF	; line 308

PatTab7
	addwf	PCL,f	; line 309
	retlw	0x17	; line 310
	retlw	0x0B	; line 311
	retlw	0x17	; line 312
	retlw	0x0B	; line 313
	retlw	0x19	; line 314
	retlw	0x0D	; line 315
	retlw	0x19	; line 316
	retlw	0x0D	; line 317
	retlw	0x1A	; line 318
	retlw	0x0E	; line 319
	retlw	0x1A	; line 320
	retlw	0x0E	; line 321
	retlw	0x1C	; line 322
	retlw	0x10	; line 323
	retlw	0x1C	; line 324
	retlw	0x10	; line 325
	retlw	0x19	; line 326
	retlw	0x0D	; line 327
	retlw	0x19	; line 328
	retlw	0x0D	; line 329
	retlw	0x15	; line 330
	retlw	0x09	; line 331
	retlw	0x15	; line 332
	retlw	0x09	; line 333
	retlw	0xFF	; line 334

PatTab8
	addwf	PCL,f	; line 335
	retlw	0x17	; line 336
	retlw	0x0B	; line 337
	retlw	0x17	; line 338
	retlw	0x0B	; line 339
	retlw	0x1C	; line 340
	retlw	0x10	; line 341
	retlw	0x1C	; line 342
	retlw	0x10	; line 343
	retlw	0x19	; line 344
	retlw	0x0D	; line 345
	retlw	0x19	; line 346
	retlw	0x0D	; line 347
	retlw	0x15	; line 348
	retlw	0x09	; line 349
	retlw	0x15	; line 350
	retlw	0x09	; line 351
	retlw	0x1A	; line 352
	retlw	0x15	; line 353
	retlw	0x12	; line 354
	retlw	0x0E	; line 355
	retlw	0x09	; line 356
	retlw	0x06	; line 357
	retlw	0x02	; line 358
	retlw	0x00	; line 359
	retlw	0xFF	; line 360

	end
